#!/usr/bin/env python

#     Copyright (C) 2002 Stas Z. <stas@linux.isbeter.nl>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
#
"""  Childsplay - A suite of educational games for young children.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
    
  This is beta software, please send bugs and/or comments to
  childsplay@users.sourceforge.net"""


import os,sys,random,fnmatch,gettext,locale,pickle,operator
import pygame
from pygame.constants import RLEACCEL,KEYDOWN,K_BACKSPACE,K_RETURN,MOUSEBUTTONDOWN,QUIT,K_ESCAPE,K_F1,FULLSCREEN
from utils import *

fullscr = 1 #set to toggle fullscreen
version = '0.66'
helptext = "Not much help here yet, look at the manpage and docs."
if len(sys.argv) > 1:
    for item in sys.argv[1:]:
        if item == '--fullscreen':
            fullscr = 1
        elif item == '--window':
            fullscr = None
        elif item == '--version':
            print version
            raise SystemExit
        elif item == '--help':
            print helptext
            raise SystemExit
    
print >> sys.stderr,__doc__

basedir = os.path.dirname(sys.argv[0])
libdir = os.path.join(basedir,'lib')
if sys.platform == 'win32':
    localedir = os.path.join(basedir,'locale')
else:
    localedir = os.path.join('/','usr','share','locale')
sys.path.append((libdir))
# i18n stuff 
# XXX Now win98 is broken, how to set i18n on win??
stdloc = locale.setlocale(locale.LC_ALL,'')

try:
    gettext.install('childsplay',localedir)
except IOError,info:
    trace_error()
    print >> sys.stderr, "Can't find i18n files for",stdloc
    print >> sys.stderr, "Using standard, -> english"
    import __builtin__
    __builtin__.__dict__['_'] = gettext.gettext

# check and load modules
modules = []
for name in get_files(os.path.abspath(libdir),'*.py'): # from utils.py
    if os.path.exists(os.path.join(basedir,'Data','icons',name[:-3]+'.icon.png')):
        modules.append((name[:-3]))
    else:
        print >> sys.stderr,'Can\'t find menu icon for',name,'\nRemoving',name,'from modules list'

menudic = {}
rows = 1+len(modules)/6
y = 80
x = 80
for row in range(rows):
    for name in modules:
        menudic[(x,y,80,80)] = (name,name+'.icon.png',1)
        x += 100
    y += 100
    x = 80

class Img:
    pass

class Snd:
    pass
# XXX not in use yet
class Mouse:
    def __init__(self):
        start = pygame.display.get_surface().get_size()
        self.pointer = load_image(os.path.join(basedir,'Data','pointer.png'),1)
        self.rect = self.pointer.get_rect()
        self.rect.move_ip(300,300)
                
    def update(self):
        pos = pygame.mouse.get_pos()
        self.rect.centerx,self.rect.centery = pos
        return 0

class Firework:
    def __init__(self,x,letter):
        self.sndboom = Snd.boom
        
        self.fireR = Img.fireR
        self.fireL = Img.fireL
        self.img = self.fireR
        self.letter = letter
        self.x = x
        self.y = 600
        self.speed = 10
        self.height = 100
        self.rect = self.fireL.get_rect()
        self.rect.move_ip(self.x,self.y)
        
    def update(self):
        if self.y <= self.height:
            self.y = 600
            self.sndboom.play()
            self.img = self.letter
            return 1
        else:
            self._move()
        return 0
    
    def _move(self):
        self.y -= self.speed
        self.rect.move_ip(0,-self.speed)
        if self.img is self.fireR:
            self.img = self.fireL
        else:
            self.img = self.fireR

class Menu:
    def __init__(self,screen,backcol,menudic):
        self.screen  =screen
        self.backcol = backcol
        self.menudic = menudic
        
        self.menuitems = []
        self.menurects = []
        self._setup()
                
    def _setup(self):
        class MenuItem:
            pass
        
        for key in self.menudic.keys():
            obj = MenuItem()
            obj.img = load_image(os.path.join(basedir,'Data','icons',self.menudic[key][1]),self.menudic[key][2])
            r = obj.img.get_rect()
            obj.rect = r.move(key[:2])
            self.menuitems.append((obj))
            self.menurects.append((obj.rect))
            
    def show(self):
        dirtyrects = []
        for obj in self.menuitems:
            dirtyrects.append((self.screen.blit(obj.img,obj.rect)))
        pygame.display.update(dirtyrects)
    
    def helptitle(self):
        return "Childsplay"    
    
    def help(self):
        text = [_("A suite of educational games for young children."),
_("It's using a plug-in system for the games, so childsplay is basically a"),
_("framework where you put in as many games as you like."), 
_("Childsplay comes with a few standard games, so you might want to check"),
_("the website (see below) for additional games."),
" ",
_("All the games have there own help page, just start a game and hit the"),
_("questionmark again"),
" ",
_("Childsplay and all the games are licensed with the GNU-GPL."),
_("All the sounds and pictures as well as the fonts used by childsplay and the games"),
_("are free in the public domain or used with permission of the owner."),
_("If you think that something is used without permission or you have a "),
_("'See-you-in-court' copyright on it, please let me know and i will remove it"),
" ",
_("For updates and info: http://childsplay.sourceforge.net"),
_("For additional game-modules : http://childsplay.sourceforge.net/plugins.html"),
_("Comments, bugs, ideas and other things of interest, send email to:"),
_("childsplay@users.sourceforge.net"),
" ",
_("Credits:"),
"Some images  2002 www.clipart.com",
"TrueType font by www.larabiefonts.com (freeware ttf's)",
" ",
"Thanks:",
"French translation by Bruno.Coudoin@free.fr",
"Danish translation by Gunner, gp@lyngbjerggaardskolen.dk"]
        
        return text
    
    def loop(self,events):
        for event in events:
            if event.type is MOUSEBUTTONDOWN:
                pos = pygame.Rect(pygame.mouse.get_pos() + (4,4))
                for r in self.menurects:
                    if r.contains(pos):
                        Snd.music.stop()
                        return tuple(r),0
        return 0,0
      
class ScoreThing:
    """ Provides a score surface"""
    def __init__(self,fsize=12,fcol=(0,0,0)):
        self.fsize = fsize
        self.fcol = fcol
        surf_s,s_size = font2surf('Score:',self.fsize,self.fcol,None,1)
        self.score_pos = (12,4+s_size[1])
        surf_size = (s_size[0]+12,s_size[1]*2+4)
        self.base_surf = pygame.Surface(surf_size).convert()
        pygame.draw.rect(self.base_surf,(0,255,0),self.base_surf.get_rect(),4)
        self.base_surf.blit(surf_s,(6,4))
        #print self.base_surf
    
    def update(self,score='0000'):
        surf = self.base_surf.convert()
        sc = ('0000'+score)[-4:]# left fill with zero's
        surf_s,s_size = font2surf(sc,self.fsize,self.fcol,None,1)
        surf.blit(surf_s,self.score_pos)
        return surf
        
class Score:
    def __init__(self):
        self.hsc_ttf = os.path.join(basedir,'Data','bullpen3.ttf')
        self.sc_ttf = os.path.join(basedir,'Data','bluebold.ttf')
        self.sc_fsize = 32
        self.sc_fcol = (7,249,23)
        self.bal_l_img = load_image(os.path.join(basedir,'Data','balloonsl.png'),1)
        self.bal_r_img = load_image(os.path.join(basedir,'Data','balloonsr.png'),1)
        try:
            f = open(os.path.join(basedir,'Data','score'),'r')
            self.score_sheet = pickle.load(f)
            f.close()
        except (EOFError,IOError),info:
            trace_error()
            print >> sys.stderr,'Something went wrong with the score file\n Trying to create a new one'
            self.score_sheet = self._make_default_sheet()
            try:
                self._put_sheet()
                print >> sys.stderr, 'OK'
            except MyError:
                print >> sys.stderr,'Problems with the scoring class, using bogus scoring',\
                info,'\n',MyError.name,MyError.line,'\nPlease send a bug report with the above stuff to\n childsplay@users.sourceforge.net'

    
    def _make_default_sheet(self):
        sheet = {}
        templ = []
        for name in ['Tux','Linus','Guido','RMS','Dijkstra']:
            templ.append(((100,name)))
        templ.sort()
        for name in modules:
            sheet[name.capitalize()] = templ[:]
        return sheet

    def _fetch_score(self):
        try:
            f = open(os.path.join(basedir,'Data','score'),'r')
            score_sheet = pickle.load(f)
            f.close()
        except (IOError,EOFError):
            trace_error()
            print >> sys.stderr,'Error in score class, don\'t use it','Using bogus sheet'
            return self._make_default_sheet()
        return score_sheet
            
    def _put_sheet(self):
        try:
            f = open(os.path.join(basedir,'Data','score'),'w')
            pickle.dump(self.score_sheet,f)
            f.close()
        except IOError,info:
            trace_error()
            print >> sys.stderr,'Error in writing score'
            MyError.name = 'Error in writing score'
            MyError.line = info
            raise MyError
    
    def get_score(self,name):
        self.score_sheet = self._fetch_score()
        try:
            sheet = self.score_sheet[name]
        except KeyError:
            return [(0,'Not possible to score points here')]
        sheet.sort()
        sheet.reverse() #list items -> (score,name)
        return sheet

    def check_high_score(self,name,score):
        sheet = self.get_score(name)
        flag = None
        if score >= sheet[-1][0]:
            flag = 1
            if len(self.score_sheet[name]) >= 8:
                self.score_sheet[name] = self.score_sheet[name][:7]
                self._put_sheet()
            self.score_sheet[name] = self._high_score(name,score)
            self._put_sheet()
        self.clear_scr()
        return flag #trigger game over screen or not
    
    def _get_key(self):
        event = pygame.event.poll()
        if event.type is KEYDOWN:
            if 96<event.key<123 :
                surf,spam = font2surf(chr(event.key),self.sc_fsize,self.sc_fcol,self.hsc_ttf)
                return surf,chr(event.key)
            elif event.key is K_BACKSPACE:
                return None,'back'
            elif event.key is K_RETURN:
                return None,'end'
        return None,None
    
    def _high_score(self,name,score):
        
        sheet = self.get_score(name)
        self.sdim.dim(darken_factor=255,color_filter=(0,0,0))
        temp_back = pygame.Surface((800,500))
        temp_back.blit(self.screen,(0,0),(0,0,800,500))
        #self._score_eyecandy(temp_back)
        pygame.display.update(self.screen.blit(self.bal_l_img,(0,0)))
        pygame.display.update(self.screen.blit(self.bal_r_img,(665,0)))
        
        file = os.path.join(basedir,'Data','Winner2.ogg')
        load_music(file).play()
                
        txt = _("Congratulations,")
        surf,spam = font2surf(txt,48,self.sc_fcol,self.hsc_ttf)
        x = 150
        y=20
        pygame.display.update(self.screen.blit(surf,(x,y)))
        
        txt = _("You are a winner")
        surf,spam = font2surf(txt,32,self.sc_fcol,self.hsc_ttf)
        x = 200
        y=120
        pygame.display.update(self.screen.blit(surf,(x,y)))
        
        txt = str(score)+_(" Points !!")
        surf,spam = font2surf(txt,48,self.sc_fcol,self.hsc_ttf)
        x = 200
        y=200
        pygame.display.update(self.screen.blit(surf,(x,y)))
        
        txt = _("Please give your name: ")
        surf,spam = font2surf(txt,self.sc_fsize,self.sc_fcol,self.hsc_ttf)
        x = 20
        y=350
        pygame.display.update(self.screen.blit(surf,(x,y)))
        x += surf.get_width()
        offset = 0
        text = ''
        surf = None
        temp_back.blit(self.screen,(0,0),(0,0,800,500))
        while len(text) < 10:
            surf, key = self._get_key()
            if not surf and not key: continue
            #print key
            if key != 'end' and key != 'back':
                old_pos = self.screen.blit(surf,(x+offset,y))
                pygame.display.update(old_pos)
                offset += surf.get_width()
                text = text+key
            elif key == 'back' and len(text) > 0:
                text = text[:-1]
                if len(text) != 0:
                    surf,spam = font2surf(text,self.sc_fsize,self.sc_fcol,self.hsc_ttf)
                    offset = surf.get_width()
                else:
                    offset = 0
                    pygame.display.update(self.screen.blit(temp_back,(x,y),(x,y-4,300,100)))
                    continue
                #print x,y,temp_back
                pygame.display.update(self.screen.blit(temp_back,(x,y),(x,y-4,300,100)))
                pygame.display.update(self.screen.blit(surf,(x,y)))
            elif key == 'end':
                break
        #print text
        sheet.append(((score,text)))
        sheet.sort()
        sheet.reverse()
        return sheet
        
class Contr(Score):
    def __init__(self):
        pygame.init()
        self.clock = pygame.time.Clock()
        file = os.path.join(basedir,'Data','chpl-icon-48.xpm')
        try:
            surface = pygame.image.load(file)
        except pygame.error:
            trace_error()
            print >> sys.stderr,'Could not load image "%s"\n %s'%(file, pygame.get_error())
        surface.set_colorkey(surface.get_at((0, 0)), RLEACCEL)
        pygame.display.set_icon(surface)
        #pygame.display.set_icon(load_image(os.path.join(basedir,'Data','chpl-icon-56.bmp'),1))
        self.sdim = Dimmer(keepalive=0)
        self.tuxplace = 20
        self.tuxrects = []
        self.mainmenurects = []
        self.setup()
        # fire up the score stuff
        Score.__init__(self)
                
    def setup(self):
        x = 800
        y = 600
        self.y = y
        self.x = x
        if fullscr:
            self.screen = pygame.display.set_mode((x, y),FULLSCREEN)
        else:
            self.screen = pygame.display.set_mode((x, y))
        
        pygame.display.set_caption(_('Childsplay, educational games for young children'))
        self.backcol = (80,151,244)
        self.ttf = os.path.join(basedir,'Data','bluehigh.ttf')
        self.backgr = pygame.Surface((x,y))
        Img.back = load_image(os.path.join(basedir,'Data','blue.jpg'),0)
        self.backgr.blit(Img.back, (0,0))
        #self.backgr.fill(self.backcol)
        # Build screen
        Img.land = load_image(os.path.join(basedir,'Data','land.png'),1)
        self.backgr.blit(Img.land, (0,y-89))
        Img.stop  = load_image(os.path.join(basedir,'Data','stop.png'),1)
        self.mainmenurects.append((self.backgr.blit(Img.stop, (700,y-90))))
        Img.qmark  = load_image(os.path.join(basedir,'Data','question1.png'),1)
        self.mainmenurects.append((self.backgr.blit(Img.qmark, (32,y-96))))
        Img.hscore = load_image(os.path.join(basedir,'Data','hscore.png'),1)
        self.mainmenurects.append((self.backgr.blit(Img.hscore, (120,y-85))))
        
        self.scorething = ScoreThing(24,(0,255,0))
        self.scorething_place = (600,y-60)
        self.backgr.blit(self.scorething.update(),self.scorething_place)
        
        Img.tux = load_image(os.path.join(basedir,'Data','tux.png'),1)
               
        Img.OK = load_image(os.path.join(basedir,'Data','OK-but.png'),1)
        Img.MORE = load_image(os.path.join(basedir,'Data','MORE-but.png'),1)
        
        self.screen.blit(self.backgr, (0, 0))
        Img.screen = self.screen
        pygame.display.update()
        
        #self.mouse = Mouse()
        
        file = os.path.join(basedir,'Data','level_clear.wav')
        Snd.levelclear = load_sound(file) 
        
        # Stuff for the fireworks class (game over)
        file = os.path.join(basedir,'Data','boom.wav')
        Snd.boom = load_sound(file)
        file = os.path.join(basedir,'Data','rocket.wav')
        Snd.rocket = load_sound(file)
        
        Img.fireR = load_image(os.path.join(basedir,'Data','firework1.png'),1)
        Img.fireL = load_image(os.path.join(basedir,'Data','firework2.png'),1)
        Img.black = pygame.Surface(Img.fireL.get_size()).convert()
        
    def mainloop(self,obj):
        stop,score = 0,0
        self.score = 0
        # stop and score flags
        # stop -> 0=continue, 1=like exit (see main), -1=clear screen, 2=exit (game or main exit)
        while stop == 0:
            self.clock.tick(60)
            events = pygame.event.get()
            
            #old_pos = self.screen.blit(self.backgr,self.mouse.rect,self.mouse.rect)
            #self.mouse.update()
            #r = self.screen.blit(self.mouse.pointer,self.mouse.rect)
            #pygame.display.update((old_pos,r))
            
            ### run game loop ##
            stop,score = obj.loop(events[:])
            
            if score:
                #print 'c.mainloop stop',stop,'score',score
                self.score += score
                if game_score >= 0 and self.score > -1:
                    self.display_scorething(str(game_score+self.score))
                score = 0
            for event in events:
                if event.type is QUIT:
                    self.ask_exit()
                elif event.type is KEYDOWN and event.key is K_ESCAPE:
                    self.ask_exit()
                # THIS IS ONLY FOR TAKING SCREENSHOTS
                # IT SHOULD BE COMMENT OUT FOR RELEASE
                #elif event.type is KEYDOWN and event.key == K_F1:
                #    try:
                #        scr = pygame.display.get_surface()
                #        print scr
                #        pygame.image.save(scr,'/tmp/sdlscreen.BMP')
                #    except RuntimeError,info:
                #        print >> sys.stderr, info
                # END OF SCREENSHOT STUFF
                elif event.type is MOUSEBUTTONDOWN:
                    pos = pygame.Rect(pygame.mouse.get_pos() + (4,4))
                    for r in self.mainmenurects:
                        if r.contains(pos):
                            #print r.topleft
                            if r.topleft[0] == 32:
                                stop = self.get_help(obj)
                            elif r.topleft[0] == 120 and obj.__class__.__name__ == 'Game':
                                #OK show score
                                stop = self.show_score(obj.helptitle())
                            elif r.topleft[0] == 700:
                                if obj.__class__.__name__ == 'Menu':
                                    stop = self.ask_exit()# if return 0 then no exit    
                                else:
                                    stop = 2
        return stop,self.score
        
    def ask_exit(self):
        oldclip = self.screen.get_clip()
        self.screen.set_clip()
        text = [_("Do you really want to quit ?"),_("Hit (Y)es or (N)o")]
        dirtyrects = []
        self.sdim.dim(darken_factor=180,color_filter=(51,10,10))
        #sdim.dim(darken_factor=200,color_filter=(9,247,19)) # Used for testing
        
        x= 100
        y= 200
        fcol = (255,0,0)
        #fcol = (224,109,67) # Used for testing
        for line in text:
            surf,size = font2surf(line,56,fcol,self.ttf)
            dirtyrects.append((self.screen.blit(surf,(x,y))))
            y += surf.get_height() + 50
        pygame.display.update(dirtyrects)
        
        stop = 0
        while stop == 0:
            events = pygame.event.get()
            for event in events:
                if event.type is KEYDOWN and event.key == ord(_("y").lower()):
                    raise SystemExit
                elif event.type is KEYDOWN and not event.key == ord(_("y").lower()):
                    stop = 1
        self.sdim.undim()
        self.screen.set_clip(oldclip)
        return 0
            
    def display_scorething(self,score):
        if score < 0: return
        self.screen.set_clip()
        self.backgr.set_clip()
        surf = self.scorething.update(score)
        pygame.display.update(self.screen.blit(surf,self.scorething_place))
        self.backgr.blit(surf,self.scorething_place)
        self.screen.set_clip(0,0,800,500)
        self.backgr.set_clip(0,0,800,500)
            
    def show_score(self,name):
        class spam:
            """ Bogus class to get a class obj for the help function"""
            def __init__(self,name,txt):
                self.name = name
                self.txt = txt
            def helptitle(self):
                return self.name
            def help(self):
                return self.txt
        txt = []
        i = 1
        for item in self.get_score(name):
            txt.append(("               %s  %s%s%s" % (i,item[1]," - ",item[0])))
            i += 1
        obj = spam(name,txt)
        self.get_help(obj,32)
        return 0
    
    def clear_scr(self,backgr=None):
        self.screen.blit(screendim.buffer,(0,0))
        if backgr:
            self.backgr.blit(screendim.buffer,(0,0))
        screendim.undim()
    
    def place_tux(self):
        Snd.levelclear.play()
        screendim.undim()
        for r in self.tuxrects:
            pygame.display.update(self.screen.blit(Img.tux, r))
        self.tuxplace = self.tuxplace+ 80
        r = self.screen.blit(Img.tux, (120 + self.tuxplace,self.y-94))
        self.backgr.blit(Img.tux, (120 + self.tuxplace,self.y-94))
        self.tuxrects.append((r))
        pygame.display.update(r)
       
        pygame.time.wait(1000)
    
    def get_help(self,obj,fsize=24):
        
        helptxt = obj.help()
        if not helptxt:
            return 0
        moretxt = None # text and flag for second screen
        if len(helptxt) > 15: # max 15 lines of text by font size 20
            helptxt, moretxt = helptxt[:14], helptxt[14:]
            
        helptitle = obj.helptitle()
        update_rects = []
        oldclip = self.screen.get_clip()
        self.screen.set_clip()
        self.sdim.dim(darken_factor=180,color_filter=(51,10,10))
        
        surf = txt2surf(helptxt,self.x-20,self.y-104,fsize,helptitle,1,(158,196,107),self.ttf)
        update_rects.append((self.screen.blit(surf,(0,0))))
        if moretxt:
            okrect = self.screen.blit(Img.OK,(self.x-700,self.y-90))
            update_rects.append((okrect))
            morerect = self.screen.blit(Img.MORE,(self.x-400,self.y-90))
            update_rects.append((morerect))
        else:
            okrect = self.screen.blit(Img.OK,(self.x-500,self.y-90))
            update_rects.append((okrect))
            morerect = None
        pygame.display.update((update_rects))
        
        stop = 0
        while stop == 0:
            events = pygame.event.get()
            for event in events:
                if event.type is MOUSEBUTTONDOWN:
                    pos = pygame.Rect(pygame.mouse.get_pos() + (4,4))
                    if okrect.contains(pos):
                        stop = 1
                    elif morerect:
                        if morerect.contains(pos):
                            self.sdim.undim()
                            self.sdim.dim(darken_factor=180,color_filter=(51,10,10))
                            surf = txt2surf(moretxt,self.x-20,self.y-104,fsize,helptitle,1,(158,196,107),self.ttf)
                            update_rects.append((self.screen.blit(surf,(0,0))))
                            okrect = self.screen.blit(Img.OK,(self.x-500,self.y-90))
                            update_rects.append((okrect))
                            morerect = None
                            pygame.display.update((update_rects))
        self.sdim.undim()
        self.screen.set_clip(oldclip)
        return 0
        
    def game_over(self):
        # reset counters for placing tux
        self.tuxplace = 0
        self.tuxrects = []
        self.gameoverobj = []
        x = 10
        for item in 'GAME OVER':
            letter,size = font2surf(item,96,(209,249,9),self.ttf)
            self.gameoverobj.append((Firework(x,letter)))
            x = x + size[0][1]-16
        
        self.sdim.dim(darken_factor=255,color_filter=(0,0,0))
        for obj in self.gameoverobj:
            Snd.rocket.play()
            stop = 0
            while stop == 0:
                self.clock.tick(60)
                r = self.screen.blit(Img.black,obj.rect)
                stop = obj.update()
                rr = self.screen.blit(obj.img,obj.rect)
                pygame.display.update((r,rr))
            pygame.time.wait(500)
        pygame.time.wait(1000)
        self.sdim.undim()
        self.clear_scr(1)
        
        

def main():
    global screendim,game_score
    c  = Contr()
    modul = 'Menu'
    screendim = Dimmer(keepalive=1) # dimmer is used for screen copying
    screendim.dim(darken_factor=0) # just stores a copy in Dimmer.buffer
    screen  = c.screen
    backcol = c.backcol
    menu = Menu(screen,backcol,menudic)
    
    while 1:
        file = os.path.join(basedir,'Data','music1.ogg')
        Snd.music = load_music(file)
    
        Snd.music.play(-1)
        menu.show()
        choice,spam = c.mainloop(menu)
        if choice == 2:
            raise SystemExit
        #print menudic[choice]
        c.screen.blit(screendim.buffer,(0,0))
        c.backgr.blit(screendim.buffer,(0,0))
        screendim.undim()
        
        stop = None
        try:
            # import game module
            game_module = import_module(os.path.join(libdir,menudic[choice][0])+'.pyc')
            #impmod = "import %s" % menudic[choice][0]
            #exec impmod
            # bind gettext to game domain
            try:
                i18n = "gettext.install('%s','%s')" % (menudic[choice][0],localedir)
                exec i18n
            except IOError,info:
                #trace_error()
                print >> sys.stderr, "Can't find i18n files for",stdloc
                print >> sys.stderr, "Using standard, -> english"
                import __builtin__
                __builtin__.__dict__['_'] = gettext.gettext
            
            #inst = "%s.%s" % (menudic[choice][0],'Game')
            #obj = eval(inst)
            obj = game_module.Game
            c.screen.set_clip(0,0,800,500)
            c.backgr.set_clip(0,0,800,500)
            game = apply(obj,(c.screen,c.backgr,backcol,basedir,libdir))
            c.screen.set_clip()
            c.backgr.set_clip()
            print game.__doc__
            print game.start.__doc__
            helptitle = game.helptitle()
            levels = game.gamelevels
            items = game.gameitems
            game_score = 0
            for l in levels:
                c.display_scorething(str(game_score))
                for i in items: 
                    #print 'i',i
                    stop = 0
                    c.screen.set_clip(0,0,800,500)
                    c.backgr.set_clip(0,0,800,500)
                    game.start(l,i)
                    stop,score = c.mainloop(game)
                    if score and game_score > -1:
                        game_score += score
                        c.display_scorething(str(game_score))
                        #print 'score',score,'game_score',game_score
                    if stop == -1:
                        stop = 0
                        c.screen.blit(screendim.buffer,(0,0))
                        c.backgr.blit(screendim.buffer,(0,0))
                        screendim.undim()
                    elif stop == 2 or stop == 1:
                        del game
                        # reset counters for placing tux
                        c.tuxplace = 20
                        c.tuxrects = []
                        c.screen.set_clip()
                        c.backgr.set_clip()
                        break
                    c.screen.set_clip()
                    c.backgr.set_clip()
                else:
                    c.place_tux()
                
                if stop == 2 or stop == 1:
                    break
        except MyError,info:
            trace_error()
            print >> sys.stderr,'********************** ERROR *********************\n',\
                    ' A error occured in module',MyError.name,' LineNo:',MyError.line,'\n',\
                    ' The error message is:\n',info,\
                    '\n**************************************************'
            stop = 2
        try:
            gettext.install('childsplay',localedir)
        except IOError:
            import __builtin__
            __builtin__.__dict__['_'] = gettext.gettext
        if stop != 2:
            if game_score:
                c.check_high_score(helptitle,game_score)
                game_score = 0
            c.game_over()
        c.screen.set_clip()
        c.backgr.set_clip()
        c.screen.blit(screendim.buffer,(0,0))
        c.backgr.blit(screendim.buffer,(0,0))
        screendim.undim()
        
        
if __name__ == '__main__':
    main()
    locale.setlocale(locale.LC_ALL,stdloc)